@@ -1,6 +1,7 @@
 import random
 
 from jaaql.db.db_interface import DBInterface, RET__rows
+from jaaql.email.email_manager_service import EmailAttachment
 from jaaql.mvc.base_model import BaseJAAQLModel, CONFIG_KEY__security, CONFIG_KEY_SECURITY__mfa_label, \
     CONFIG_KEY_SECURITY__mfa_issuer, VAULT_KEY__jwt_obj_crypt_key, VAULT_KEY__jwt_crypt_key, DIR__apps, SEPARATOR__dir
 from jaaql.exceptions.http_status_exception import HttpStatusException, HTTPStatus, ERR__connection_expired, \
@@ -15,6 +16,7 @@ from jaaql.mvc.response import JAAQLResponse
 from collections import Counter
 from os.path import dirname
 from jaaql.utilities import crypt_utils
+from flask import send_file
 import uuid
 import os
 import json
@@ -31,12 +33,17 @@ import threading
 
 TOKEN__pre_auth_reduction_factor = 15
 
+ERR__document_created_file = "Document is a file, cannot be downloaded in this way"
+ERR__as_attachment_unexpected = "Input 'as_attachment' unexpected as document is returned as a file link"
 ERR__incorrect_invite_code = "Incorrect invite code"
+ERR__incorrect_reset_code = "Incorrect reset code"
 ERR__invite_code_expired = "Invite code expired. Please use the link within the email"
+ERR__reset_code_expired = "Reset code expired. Please use the link within the email"
 ERR__not_signed_up = "Not signed up"
 ERR__recipient_not_allowed = "Recipient not allowed"
 ERR__cant_send_attachments = "Cannot send attachments to other people"
 ERR__template_not_signup = "One of the supplied templates is not suitable for signup"
+ERR__template_reset_password = "You cannot use this template to reset your password"
 ERR__please_sign_in = "Please sign in, this user already exists and is signed up!"
 ERR__incorrect_install_key = "Incorrect install key!"
 ERR__incorrect_credentials = "Incorrect credentials!"
@@ -56,11 +63,11 @@ ERR__user_does_not_exist = "The user does not exist, cannot resend the email"
 ERR__data_validation_table_no_primary = "Data validation table has no primary key"
 ERR__uninstallation_not_allowed = "Uninstallation not allowed"
 ERR__password_required = "Password required"
-ERR__cant_find_sign_up = "Cannot locate sign up with key. The key is either incorrect or has expired"
+ERR__cant_find_sign_up = "Cannot locate sign up with key. The key is either incorrect, has expired or has not been activated with the emailed code"
+ERR__cant_find_reset = "Cannot locate reset with key. The key is either incorrect, has expired or has not been activated with the emailed code"
 
 SQL__err_duplicate_user = "duplicate key value violates unique constraint \"jaaql__user_unq_email\""
 
-USERNAME__jaaql = "jaaql"
 USERNAME__superjaaql = "superjaaql"
 USERNAME__postgres = "postgres"
 
@@ -71,10 +78,23 @@ CONFIGURATION__host = "host"
 
 FUNC__jaaql_create_node = "jaaql__create_node"
 
-CODE__all_chars = "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789"
+CODE__letters = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
+CODE__alphanumeric = "ABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890"
+CODE__invite_length = 5
+CODE__reset_length = 8
+CODE__max_attempts = 3
+RESEND__invite_max = 2
+RESEND__invite_not_registered_max = 3
+RESEND__reset_max = 2
 
+CONFIG__default = "Default config"
+CONFIG__default_desc = "Default config description"
+DATASET__default = "Default dataset"
+DATASET__default_desc = "Default dataset description"
+
+ATTR__count = "count"
 ATTR__ip_id = "ip_id"
-ATTR__expiry_invite_code_ms = "code_expiry_ms"
+ATTR__expiry_code_ms = "code_expiry_ms"
 ATTR__used_key_a = "used_key_a"
 ATTR__email = "email"
 ATTR__public_credentials = "public_credentials"
@@ -89,6 +109,7 @@ ATTR__alias = "alias"
 ATTR__data_lookup_json = "data_lookup_json"
 ATTR__activated = "activated"
 ATTR__closed = "closed"
+ATTR__code_attempts = "code_attempts"
 KEY__totp_iv = "totp_iv"
 KEY__user_id = "user_id"
 KEY__occurred = "occurred"
@@ -129,7 +150,6 @@ JWT__ip = "ip"
 PG__default_connection_string = "postgresql://postgres:%s@localhost:5432/jaaql"
 
 DIR__scripts = "scripts"
-DIR__www = "www"
 DIR__manager = "manager"
 DIR__playground = "playground"
 DIR__console = "console"
@@ -146,6 +166,10 @@ SIGNUP__started = 1
 SIGNUP__already_registered = 2
 SIGNUP__completed = 3
 
+RESET__not_started = 0
+RESET__started = 1
+RESET__completed = 2
+
 
 class JAAQLModel(BaseJAAQLModel):
 
@@ -205,6 +229,51 @@ class JAAQLModel(BaseJAAQLModel):
         params = self.validate_deletion_key(inputs[KEY__deletion_key], DELETION_PURPOSE__email_template)
         execute_supplied_statement_singleton(jaaql_interface, QUERY__email_template_del, params)
 
+    def render_document(self, inputs: dict, oauth_token: str):
+        inputs[KEY__oauth_token] = self.refresh(oauth_token)
+        return execute_supplied_statement_singleton(self.jaaql_lookup_connection, QUERY__ins_rendered_document, inputs,
+                                                    encryption_key=self.get_db_crypt_key(), encrypt_parameters=[KEY__parameters, KEY__oauth_token],
+                                                    as_objects=True)
+
+    def fetch_document(self, inputs: dict, response: JAAQLResponse):
+        res = execute_supplied_statement_singleton(self.jaaql_lookup_connection, QUERY__fetch_rendered_document,
+                                                   {KEY__document_id: inputs[KEY__document_id]}, singleton_message=ERR__document_id_not_found,
+                                                   as_objects=True)
+
+        if not res[KEY__completed]:
+            raise HttpStatusException(ERR__document_still_rendering, HTTP_STATUS__too_early)
+
+        if res[KEY__create_file]:
+            if inputs[KEY__as_attachment] is not None:
+                raise HttpStatusException(ERR__as_attachment_unexpected)
+            response.response_code = HTTPStatus.CREATED
+            return self.url + "/" + DIR__render_template + "/" + res[KEY__document_id] + "." + res[KEY__render_as]
+        else:
+            return self.url + "/api/rendered_documents/" + res[KEY__document_id]
+
+    def fetch_document_stream(self, inputs: dict):
+        res = execute_supplied_statement_singleton(self.jaaql_lookup_connection, QUERY__fetch_rendered_document,
+                                                   {KEY__document_id: inputs[KEY__document_id]}, singleton_message=ERR__document_id_not_found,
+                                                   as_objects=True)
+
+        if not res[KEY__completed]:
+            raise HttpStatusException(ERR__document_still_rendering, HTTP_STATUS__too_early)
+
+        if res[KEY__create_file]:
+            raise HttpStatusException(ERR__document_created_file)
+        else:
+            content = execute_supplied_statement_singleton(self.jaaql_lookup_connection, QUERY__purge_rendered_document,
+                                                           {KEY__document_id: inputs[KEY__document_id]}, as_objects=True)[KEY__content]
+            as_attachment = False
+            if inputs[KEY__as_attachment]:
+                as_attachment = True
+
+            buffer = BytesIO()
+            buffer.write(content)
+            buffer.seek(0)
+
+            return send_file(buffer, as_attachment=as_attachment, attachment_filename=res[KEY__filename])
+
     def verify_user(self, username: str, ip_address: str):
         username = username.lower()
 
@@ -492,7 +561,7 @@ class JAAQLModel(BaseJAAQLModel):
             raise HttpStatusException(ERR__already_installed)
 
     def log(self, user_id: str, occurred: datetime, duration_ms: int, exception: str, contr_input: str, ip: str, status: int,
-                 endpoint: str, databases: list = None):
+            endpoint: str, databases: list = None):
         threading.Thread(target=self.log_sync, args=[user_id, occurred, duration_ms, exception, contr_input, ip, status, endpoint, databases]).start()
 
     def log_sync(self, user_id: str, occurred: datetime, duration_ms: int, exception: str, contr_input: str, ip: str, status: int, endpoint: str,
@@ -562,8 +631,9 @@ class JAAQLModel(BaseJAAQLModel):
         return create_interface(self.config, auth[KEY__address], auth[KEY__port], DB__jaaql, auth[KEY__username], auth[KEY__password]
                                 ), user[KEY__id], ip_id, iv, user[ATTR__password_hash], last_totp, username, user[KEY__is_public]
 
-    def add_application(self, inputs: dict, jaaql_connection: DBInterface, ip_address: str, response: JAAQLResponse):
+    def add_application(self, inputs: dict, jaaql_connection: DBInterface, ip_address: str, user_id: str, response: JAAQLResponse):
         public_username = inputs.pop(KEY__public_username)
+        default_database = inputs.pop(KEY__default_database)
         inputs[KEY__application_url] = self.replace_default_app_url(inputs[KEY__application_url])
         if inputs[KEY__default_email_signup_template] is not None:
             inputs[KEY__default_email_signup_template] = execute_supplied_statement_singleton(jaaql_connection, QUERY__fetch_email_template_by_name, {
@@ -574,6 +644,10 @@ class JAAQLModel(BaseJAAQLModel):
                 jaaql_connection, QUERY__fetch_email_template_by_name, {
                     KEY__template: inputs[KEY__default_email_already_signed_up_template]
                 }, as_objects=True)[KEY__id]
+        if inputs[KEY__default_reset_password_template] is not None:
+            template_inputs = {KEY__template: inputs[KEY__default_reset_password_template]}
+            inputs[KEY__default_reset_password_template] = execute_supplied_statement_singleton(jaaql_connection, QUERY__fetch_email_template_by_name,
+                                                                                                template_inputs, as_objects=True)[KEY__id]
         execute_supplied_statement(jaaql_connection, QUERY__application_ins, inputs)
         if public_username is not None:
             password = str(uuid.uuid4())
@@ -589,6 +663,28 @@ class JAAQLModel(BaseJAAQLModel):
             }
             self.make_user_public(mup_inputs, jaaql_connection)
 
+        if default_database is not None:
+            default_node = default_database.split("/")[0] if "/" in default_database else NODE__host_node
+
+            try:
+                self.add_database({KEY__create: True, KEY__node: default_node, KEY__database_name: default_database}, jaaql_connection, user_id)
+            except:
+                pass  # Sometimes the database may already exist
+
+            self.add_application_configuration({KEY__configuration_name: CONFIG__default, KEY__description: CONFIG__default_desc,
+                                                KEY__application: inputs[KEY__application_name]}, jaaql_connection)
+            self.add_application_dataset({KEY__dataset_name: DATASET__default, KEY__description: DATASET__default_desc,
+                                          KEY__application: inputs[KEY__application_name]}, jaaql_connection)
+            self.add_database_assignment({KEY__application: inputs[KEY__application_name], KEY__configuration: CONFIG__default,
+                                          KEY__database: default_database, KEY__node: default_node, KEY__dataset: DATASET__default}, jaaql_connection)
+
+            if public_username is not None:
+                execute_supplied_statement(jaaql_connection, QUERY__node_credentials_set_public_user, {KEY__username: public_username,
+                                                                                                       KEY__node: default_node})
+                self.add_user_default_role({KEY__role: public_username}, jaaql_connection)
+                self.add_configuration_authorization({KEY__application: inputs[KEY__application_name], KEY__configuration: CONFIG__default,
+                                                      KEY__role: public_username}, jaaql_connection)
+
     def delete_application(self, inputs: dict):
         return {KEY__deletion_key: self.request_deletion_key(DELETION_PURPOSE__application, inputs)}
 
@@ -620,8 +716,7 @@ class JAAQLModel(BaseJAAQLModel):
         paging_query, where_query, where_parameters, parameters = self.construct_paging_queries(inputs)
         full_query = QUERY__application_sel + paging_query
 
-        return self.execute_paging_query(jaaql_connection, full_query, QUERY__application_count, parameters,
-                                         where_query, where_parameters)
+        return self.execute_paging_query(jaaql_connection, full_query, QUERY__application_count, parameters, where_query, where_parameters)
 
     def get_my_applications(self, jaaql_connection: DBInterface):
         return execute_supplied_statement(jaaql_connection, QUERY__fetch_my_applications, as_objects=True)
@@ -672,7 +767,7 @@ class JAAQLModel(BaseJAAQLModel):
         parameters = self.validate_deletion_key(inputs[KEY__deletion_key], DELETION_PURPOSE__node)
         execute_supplied_statement(jaaql_connection, QUERY__node_del, parameters)
 
-    def add_database(self, inputs: dict, jaaql_connection: DBInterface, user_id: int = None):
+    def add_database(self, inputs: dict, jaaql_connection: DBInterface, user_id: str = None):
         inputs_create = inputs.copy()
         if KEY__create in inputs:
             inputs.pop(KEY__create)
@@ -733,8 +828,7 @@ class JAAQLModel(BaseJAAQLModel):
 
     def delete_application_configuration_confirm(self, inputs: dict, jaaql_connection: DBInterface):
         parameters = self.validate_deletion_key(inputs[KEY__deletion_key], DELETION_PURPOSE__application_configuration)
-        execute_supplied_statement(jaaql_connection, QUERY__application_configuration_del, parameters,
-                                   as_objects=True)
+        execute_supplied_statement(jaaql_connection, QUERY__application_configuration_del, parameters, as_objects=True)
 
     def add_database_assignment(self, inputs: dict, jaaql_connection: DBInterface):
         execute_supplied_statement(jaaql_connection, QUERY__assigned_database_ins, inputs)
@@ -751,8 +845,7 @@ class JAAQLModel(BaseJAAQLModel):
 
     def remove_database_assignment_confirm(self, inputs: dict, jaaql_connection: DBInterface):
         parameters = self.validate_deletion_key(inputs[KEY__deletion_key], DELETION_PURPOSE__database_assignment)
-        execute_supplied_statement(jaaql_connection, QUERY__assigned_database_del, parameters,
-                                   as_objects=True)
+        execute_supplied_statement(jaaql_connection, QUERY__assigned_database_del, parameters, as_objects=True)
 
     def add_node_authorization(self, inputs: dict, connection: DBInterface):
         execute_supplied_statement(connection, QUERY__node_credentials_ins, inputs,
@@ -840,7 +933,7 @@ class JAAQLModel(BaseJAAQLModel):
             ATTR__data_lookup_json: template_lookup,
             ATTR__the_user: the_user,
             KEY__email_template: email_template,
-            KEY__invite_code: [CODE__all_chars[random.randint(0, len(CODE__all_chars) - 1)] for i in range(4)]
+            KEY__invite_code: "".join([CODE__letters[random.randint(0, len(CODE__letters) - 1)] for _ in range(CODE__invite_length)])
         }
         return execute_supplied_statement_singleton(jaaql_connection, QUERY__sign_up_insert, params, as_objects=True)
 
@@ -960,7 +1053,7 @@ class JAAQLModel(BaseJAAQLModel):
 
         return res
 
-    def finish_signup(self, token: str):
+    def fetch_signup(self, token: str):
         resp = execute_supplied_statement_singleton(self.jaaql_lookup_connection, QUERY__sign_up_fetch, parameters={KEY__invite_or_poll_key: token},
                                                     as_objects=True, singleton_message=ERR__cant_find_sign_up)
 
@@ -969,12 +1062,20 @@ class JAAQLModel(BaseJAAQLModel):
             template_table = execute_supplied_statement_singleton(self.jaaql_lookup_connection, QUERY__fetch_email_template,
                                                                   {KEY__id: resp[KEY__email_template]}, as_objects=True)[KEY__data_validation_table]
             res[KEY__parameters] = self.select_from_data_validation_table(template_table, json.loads(resp[ATTR__data_lookup_json]))
+
+        return res
+
+    def finish_signup(self, token: str):
+        resp = execute_supplied_statement_singleton(self.jaaql_lookup_connection, QUERY__sign_up_fetch, parameters={KEY__invite_or_poll_key: token},
+                                                    as_objects=True, singleton_message=ERR__cant_find_sign_up)
+
+        if resp[KEY__email_template] and resp[ATTR__data_lookup_json]:
+            template_table = execute_supplied_statement_singleton(self.jaaql_lookup_connection, QUERY__fetch_email_template,
+                                                                  {KEY__id: resp[KEY__email_template]}, as_objects=True)[KEY__data_validation_table]
             self.delete_from_data_validation_table(template_table, json.loads(resp[ATTR__data_lookup_json]))
 
         execute_supplied_statement(self.jaaql_lookup_connection, QUERY__sign_up_close, {KEY__invite_key: resp[KEY__invite_key]})
 
-        return res
-
     def sign_up_user(self, jaaql_connection: DBInterface, username: str, password: str, user_id: str = None, ip_address: str = None,
                      use_mfa: bool = False, response: JAAQLResponse = None, allow_password_error: bool = False):
         if user_id is None:
@@ -1007,8 +1108,7 @@ class JAAQLModel(BaseJAAQLModel):
         return resp
 
     def fetch_default_roles_as_list(self):
-        return [row[0] for row in execute_supplied_statement(self.jaaql_lookup_connection,
-                                                             QUERY__default_roles_sel)[RET__rows]]
+        return [row[0] for row in execute_supplied_statement(self.jaaql_lookup_connection, QUERY__default_roles_sel)[RET__rows]]
 
     def fetch_allowed_recipients_for_email_template(self, username: str, template: Union[str, dict], get_address: bool = False):
         if isinstance(template, str):
@@ -1090,7 +1190,7 @@ class JAAQLModel(BaseJAAQLModel):
 
         return resp
 
-    def send_email(self, inputs: dict, username: str):
+    def send_email(self, inputs: dict, username: str, oauth_token: str, user_id: str):
         if inputs[KEY__recipient] is not None and inputs[KEY__attachments] is not None:
             raise HttpStatusException(ERR__cant_send_attachments)
 
@@ -1105,17 +1205,19 @@ class JAAQLModel(BaseJAAQLModel):
         params = inputs[KEY__parameters]
         if params is not None and template[KEY__data_validation_table] is None:
             raise HttpStatusException(ERR__unexpected_parameters)
-        if inputs[KEY__data_validation_table] is not None and params is None:
+        if template[KEY__data_validation_table] is not None and params is None:
             params = {}
-        if inputs[KEY__data_validation_table] is not None:
+        if template[KEY__data_validation_table] is not None:
             params, _ = self.fetch_sanitized_email_params(template, params)
 
         allowed_recipients = self.fetch_allowed_recipients_for_email_template(username, template, True)
         if inputs[KEY__recipient] not in allowed_recipients:
             raise HttpStatusException(ERR__recipient_not_allowed)
 
-        self.email_manager.construct_and_send_email(self.url, app_url, template, username, allowed_recipients[inputs[KEY__recipient]],
-                                                    inputs[KEY__recipient], params, attachments=KEY__attachments)
+        self.email_manager.construct_and_send_email(self.url, app_url, template, user_id, allowed_recipients[inputs[KEY__recipient]],
+                                                    inputs[KEY__recipient], params,
+                                                    attachments=EmailAttachment.deserialize_list(inputs[KEY__attachments], template[KEY__id]),
+                                                    attachment_access_token=self.refresh(oauth_token))
 
     def fetch_sanitized_email_params(self, template: dict, params: dict):
         val_table = template[KEY__data_validation_table]
@@ -1136,12 +1238,13 @@ class JAAQLModel(BaseJAAQLModel):
         val_table_esc = '"%s"' % val_table
         ins_query = "INSERT INTO " + val_table_esc + " (" + cols + ") VALUES (" + ins + ") RETURNING " + pkeys
         pkey_vals = execute_supplied_statement_singleton(self.jaaql_lookup_connection, ins_query, params, as_objects=True)
-
+        pkey_vals = {key: str(val) if isinstance(val, uuid.UUID) else val for key, val in pkey_vals.items()}
         select_table = template[KEY__data_validation_view] if template[KEY__data_validation_view] is not None else val_table
         return self.select_from_data_validation_table(select_table, pkey_vals), pkey_vals
 
     def signup_status(self, inputs: dict):
-        resp = execute_supplied_statement_singleton(self.jaaql_lookup_connection, QUERY__sign_up_poll, parameters=inputs, as_objects=True,
+        select_param = {KEY__invite_or_poll_key: inputs[KEY__invite_or_poll_key]}
+        resp = execute_supplied_statement_singleton(self.jaaql_lookup_connection, QUERY__sign_up_poll, parameters=select_param, as_objects=True,
                                                     singleton_message=ERR__cant_find_sign_up)
 
         existed = False
@@ -1152,12 +1255,16 @@ class JAAQLModel(BaseJAAQLModel):
             if sub_hs.response_code != HTTPStatus.UNAUTHORIZED:
                 raise sub_hs  # Unrelated exception, raise it
 
-        is_invite_key = resp[KEY__invite_key] == inputs[KEY__invite_or_poll_key]
+        is_invite_key = str(resp[KEY__invite_key]) == inputs[KEY__invite_or_poll_key]
+
+        if not is_invite_key and not resp[ATTR__activated] and resp[ATTR__code_attempts] >= CODE__max_attempts:
+            raise HttpStatusException(ERR__too_many_code_attempts, HTTPStatus.LOCKED)
 
         status = SIGNUP__not_started
 
         invite_code_match = inputs.get(KEY__invite_code) == resp[KEY__invite_code]
-        invite_code_expired = resp[ATTR__created] + timedelta(milliseconds=resp[ATTR__expiry_invite_code_ms]) < datetime.now()
+        timezone = resp[ATTR__created].tzinfo
+        invite_code_expired = resp[ATTR__created] + timedelta(milliseconds=resp[ATTR__expiry_code_ms]) < datetime.now(timezone)
 
         if is_invite_key or resp[ATTR__activated] or (invite_code_match and not invite_code_expired):
             if invite_code_match and not resp[ATTR__activated]:
@@ -1175,6 +1282,7 @@ class JAAQLModel(BaseJAAQLModel):
         elif invite_code_match:
             raise HttpStatusException(ERR__invite_code_expired)
         else:
+            execute_supplied_statement(self.jaaql_lookup_connection, QUERY__sign_up_increment_attempts, {KEY__invite_key: resp[KEY__invite_key]})
             raise HttpStatusException(ERR__incorrect_invite_code)
 
         return {KEY__invite_key_status: status}
@@ -1209,6 +1317,11 @@ class JAAQLModel(BaseJAAQLModel):
                 user_id = execute_supplied_statement_singleton(self.jaaql_lookup_connection, QUERY__user_id_from_username,
                                                                {KEY__username: inputs[KEY__email]}, as_objects=True)[KEY__id]
 
+        attempts = execute_supplied_statement_singleton(self.jaaql_lookup_connection, QUERY__sign_up_count, {ATTR__the_user: user_id},
+                                                        as_objects=True)[ATTR__count]
+        if user_existed and attempts >= RESEND__invite_max or attempts >= RESEND__invite_not_registered_max:
+            raise HttpStatusException(ERR__too_many_signup_attempts, HTTPStatus.TOO_MANY_REQUESTS)
+
         template = execute_supplied_statement_singleton(self.jaaql_lookup_connection, QUERY__fetch_email_template_by_name, {
             KEY__email_template: inputs[KEY__email_template]
         }, as_objects=True)
@@ -1231,7 +1344,7 @@ class JAAQLModel(BaseJAAQLModel):
         pkey_vals = None
 
         if template[KEY__data_validation_table] is not None:
-            if EMAIL_PARAM__signup_key in inputs:
+            if EMAIL_PARAM__signup_key in params:
                 raise HttpStatusException(ERR__unexpected_validation_column % EMAIL_PARAM__signup_key)
 
             sanitized_params, pkey_vals = self.fetch_sanitized_email_params(template, params)
@@ -1239,11 +1352,124 @@ class JAAQLModel(BaseJAAQLModel):
         template = template_already_exists if user_existed else template
         invite_keys = self.user_invite(self.jaaql_lookup_connection, inputs[KEY__email], template[KEY__id], pkey_vals)
         optional_params = {EMAIL_PARAM__signup_key: invite_keys[KEY__invite_key], EMAIL_PARAM__invite_code: invite_keys[KEY__invite_code]}
+        optional_params = {**optional_params, **sanitized_params}
 
-        self.email_manager.construct_and_send_email(self.url, app_url, template, user_id, inputs[KEY__email], None, sanitized_params, optional_params)
+        self.email_manager.construct_and_send_email(self.url, app_url, template, user_id, inputs[KEY__email], None, {}, optional_params)
 
         return {KEY__invite_key: invite_keys[KEY__invite_poll_key]}
 
+    def send_reset_password_email(self, inputs: dict):
+        app_url = None
+        if inputs[KEY__application]:
+            app_url = execute_supplied_statement_singleton(self.jaaql_lookup_connection, QUERY__fetch_url_from_application_name,
+                                                           {KEY__application: inputs[KEY__application]}, as_objects=True)[KEY__application_url]
+        user_exists = True
+        user = None
+        try:
+            user = self.fetch_user_from_username(inputs[KEY__email], self.jaaql_lookup_connection)
+        except HttpStatusException:
+            user_exists = False
+
+        if user_exists and user[KEY__is_public]:
+            raise HttpStatusException(ERR__user_public, HTTPStatus.UNAUTHORIZED)
+
+        query = QUERY__reset_count if user_exists else QUERY__fake_reset_count
+        attempt_inputs = {ATTR__the_user: user[KEY__id]} if user_exists else {KEY__email: inputs[KEY__email]}
+        attempts = execute_supplied_statement_singleton(self.jaaql_lookup_connection, query, attempt_inputs, as_objects=True)[ATTR__count]
+        if attempts >= RESEND__reset_max:
+            raise HttpStatusException(ERR__too_many_reset_requests, HTTPStatus.TOO_MANY_REQUESTS)
+
+        template = execute_supplied_statement_singleton(self.jaaql_lookup_connection, QUERY__fetch_email_template_by_name, {
+            KEY__email_template: inputs[KEY__email_template]
+        }, as_objects=True)
+
+        if not template[KEY__allow_reset_password]:
+            raise HttpStatusException(ERR__template_reset_password)
+
+        reset_keys = {}
+        if user_exists:
+            sanitized_params = {}
+
+            params = {
+                ATTR__the_user: user[KEY__id],
+                KEY__email_template: template[KEY__id],
+                KEY__reset_code: "".join([CODE__alphanumeric[random.randint(0, len(CODE__alphanumeric) - 1)] for _ in range(CODE__reset_length)])
+            }
+            reset_keys = execute_supplied_statement_singleton(self.jaaql_lookup_connection, QUERY__reset_insert, params, as_objects=True)
+            optional_params = {EMAIL_PARAM__reset_key: reset_keys[KEY__reset_key], EMAIL_PARAM__reset_code: reset_keys[KEY__reset_code]}
+            optional_params = {**optional_params, **sanitized_params}
+
+            self.email_manager.construct_and_send_email(self.url, app_url, template, user[KEY__id], inputs[KEY__email], None, {}, optional_params)
+        else:
+            params = {
+                KEY__email: inputs[KEY__email]
+            }
+            reset_keys = execute_supplied_statement_singleton(self.jaaql_lookup_connection, QUERY__fake_reset_insert, params, as_objects=True)
+
+        return {KEY__reset_key: reset_keys[KEY__reset_poll_key]}
+
+    def reset_password_status(self, inputs: dict):
+        select_param = {KEY__reset_or_poll_key: inputs[KEY__reset_or_poll_key]}
+        was_fake = False
+
+        try:
+            resp = execute_supplied_statement_singleton(self.jaaql_lookup_connection, QUERY__reset_poll, parameters=select_param, as_objects=True,
+                                                        singleton_message=ERR__cant_find_reset)
+        except HttpStatusException as se:
+            if se.response_code == HTTPStatus.UNPROCESSABLE_ENTITY:
+                resp = execute_supplied_statement_singleton(self.jaaql_lookup_connection, QUERY__fake_reset_poll, parameters=select_param,
+                                                            as_objects=True, singleton_message=ERR__cant_find_reset)
+                was_fake = True
+            else:
+                raise se
+
+        is_reset_key = str(resp[KEY__reset_key]) == inputs[KEY__reset_or_poll_key]
+
+        if not is_reset_key and not resp[ATTR__activated] and resp[ATTR__code_attempts] >= CODE__max_attempts:
+            raise HttpStatusException(ERR__too_many_code_attempts, HTTPStatus.LOCKED)
+
+        status = RESET__not_started
+
+        reset_code_match = inputs.get(KEY__reset_code) == resp[KEY__reset_code]
+        timezone = resp[ATTR__created].tzinfo
+        reset_code_expired = resp[ATTR__created] + timedelta(milliseconds=resp[ATTR__expiry_code_ms]) < datetime.now(timezone)
+
+        if is_reset_key or resp[ATTR__activated] or (reset_code_match and not reset_code_expired):
+            if reset_code_match and not resp[ATTR__activated]:
+                execute_supplied_statement(self.jaaql_lookup_connection, QUERY__reset_upd, {KEY__reset_key: resp[KEY__reset_key]})
+            elif not resp[ATTR__used_key_a]:
+                execute_supplied_statement(self.jaaql_lookup_connection, QUERY__reset_upd_used, {KEY__reset_key: resp[KEY__reset_key]})
+
+            if resp[ATTR__closed]:
+                status = RESET__completed
+            elif resp[ATTR__activated] or resp[ATTR__used_key_a]:
+                status = RESET__started
+
+        elif reset_code_match:
+            raise HttpStatusException(ERR__reset_code_expired)
+        else:
+            if was_fake:
+                execute_supplied_statement(self.jaaql_lookup_connection, QUERY__fake_reset_increment_attempts,
+                                           {KEY__reset_key: inputs[KEY__reset_or_poll_key]})
+            else:
+                execute_supplied_statement(self.jaaql_lookup_connection, QUERY__reset_increment_attempts, {KEY__reset_key: resp[KEY__reset_key]})
+            raise HttpStatusException(ERR__incorrect_reset_code)
+
+        return {KEY__reset_key_status: status}
+
+    def reset_password_perform_reset(self, inputs: dict):
+        token = inputs[KEY__reset_key]
+        password = inputs[KEY__password]
+
+        resp = execute_supplied_statement_singleton(self.jaaql_lookup_connection, QUERY__reset_fetch, parameters={KEY__reset_or_poll_key: token},
+                                                    as_objects=True, singleton_message=ERR__cant_find_sign_up)
+        self.add_password(self.jaaql_lookup_connection, str(resp[ATTR__the_user]), password)
+        execute_supplied_statement(self.jaaql_lookup_connection, QUERY__reset_close, parameters={KEY__reset_key: token})
+
+        return {
+            KEY__email: resp[KEY__email]
+        }
+
     def create_user(self, jaaql_connection: DBInterface, username: str, db_password: str = None, mobile: str = None,
                     attach_as: str = None, precedence: int = None, roles: str = "", public_application: str = None):
         if db_password is None:
